/*
 * Botendo
 * Copyright (C) 2023 moonleay
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

package net.moonleay.botendo.extensions.music.components

import com.kotlindiscord.kord.extensions.commands.application.slash.PublicSlashCommandContext
import com.kotlindiscord.kord.extensions.components.forms.ModalForm
import dev.kord.common.Color
import dev.kord.common.entity.Snowflake
import dev.kord.core.entity.Guild
import dev.kord.rest.builder.component.ActionRowBuilder
import dev.kord.rest.builder.message.actionRow
import dev.kord.rest.builder.message.create.actionRow
import dev.kord.rest.builder.message.embed
import dev.schlaubi.lavakord.audio.Link
import dev.schlaubi.lavakord.audio.player.Player
import dev.schlaubi.lavakord.rest.loadItem
import dev.schlaubi.lavakord.rest.models.TrackResponse
import net.moonleay.botendo.util.*

object MusicManager {
    private var musicManagerMap: MutableMap<Snowflake, GuildTrackScheduler> = mutableMapOf()

    fun getGuildTrackScheduler(guild: Guild, player: Player): GuildTrackScheduler {
        return musicManagerMap.computeIfAbsent(guild.id) {
            GuildTrackScheduler(player)
        }
    }


    suspend fun addToQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String
    ) {
        addToQueue(ctx, link, search, AddType.QUEUE, false)
    }

    suspend fun upsertIntoQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String
    ) {
        addToQueue(ctx, link, search, AddType.UPSERT, false)
    }

    suspend fun addToQueue(
        ctx: PublicSlashCommandContext<LinkArguments, ModalForm>,
        link: Link,
        search: String,
        type: AddType,
        silent: Boolean
    ) {
        val player = link.player
        val item = link.loadItem(search)
        val gts = getGuildTrackScheduler(ctx.guild!!.asGuild(), player)
        val u = ctx.user.asUser()

        gts.addEvents()

        when (item.loadType) {
            TrackResponse.LoadType.TRACK_LOADED -> {
                gts.queue(item.track, type)
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} track from link"
                            this.description = "**${item.track.info.title}**\n*Queue*\nby ${item.track.info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    item.track.info.length
                                )
                            }\n" +
                                    ">>>${item.track.info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            this.thumbnail {
                                this.url =
                                    "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(item.track.info.uri) + "/maxresdefault.jpg"
                            }
                        }

                        this.actionRow(fun ActionRowBuilder.() {
                            components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        })
                    }
            }

            TrackResponse.LoadType.PLAYLIST_LOADED -> {
                val l = item.tracks.reversed()
                for (partialTrack in l) {
                    gts.queue(partialTrack, type)
                }
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} playlist from link"
                            this.description = "**${item.tracks.first().info.title}**\n*${item.playlistInfo.name}*\nby ${item.tracks.first().info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    item.tracks.first().info.length
                                )
                            }\n" +
                                    ">>>${item.tracks.first().info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            this.thumbnail {
                                this.url =
                                    "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(item.tracks.first().info.uri) + "/maxresdefault.jpg"
                            }
                        }

                        this.actionRow {
                            this.components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        }
                    }
            }

            TrackResponse.LoadType.SEARCH_RESULT -> {
                gts.queue(item.tracks.first(), type)
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0x52E01A)
                            this.title = "${type.s} track from query"
                            this.description = "**${item.tracks.first().info.title}**\n*Queue*\nby ${item.tracks.first().info.author} ;: ${
                                TimeUtil.getTimeFormatedRaw(
                                    item.tracks.first().info.length
                                )
                            }\n" +
                                    ">>>${item.tracks.first().info.uri}"
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                            this.thumbnail {
                                this.url =
                                    "https://img.youtube.com/vi/" + UrlUtil.getYtThumbnailUrl(item.tracks.first().info.uri) + "/maxresdefault.jpg"
                            }
                        }

                        this.actionRow {
                            this.components.addAll(
                                ButtonUtil.getMusicControllerButtons(
                                    player.paused,
                                    gts.repeating
                                ).components
                            )
                        }
                    }
            }

            TrackResponse.LoadType.NO_MATCHES -> {
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Not found"
                            this.description = "There were no matches."
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
            }

            TrackResponse.LoadType.LOAD_FAILED -> {
                if (!silent)
                    ctx.respond {
                        this.embed {
                            this.color = Color(0xE0311A)
                            this.title = "Load failed"
                            this.description = "There was an error while loading."
                            this.footer {
                                this.text = MessageUtil.getFooter(u)
                            }
                        }
                    }
            }
        }
    }

    enum class AddType(val s: String) {
        QUEUE("Added"),
        UPSERT("Upserted")
    }
}
